# Copyright (c) 2012-2015, 2019-2020 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

class Abort(Exception): pass
class _UnsupportedPlatform(Exception): pass

try:
    # POLLRDHUP requires linux >= 2.6.17, and its numeric value isn't
    # exposed by python so we have to copy it from /usr/include.

    import sys
    if not sys.platform.startswith('linux'):
        raise _UnsupportedPlatform
    import os
    from distutils.version import LooseVersion
    if LooseVersion(os.uname()[2]) < LooseVersion('2.6.17'):
        raise _UnsupportedPlatform

    POLLRDHUP = 0x2000

    import uwsgi
    import select

    def check_connection():
        # get the file descriptor of the request's socket from uwsgi.
        conn_fd = uwsgi.connection_fd()

        if not uwsgi.is_connected(conn_fd):
            raise Abort('uwsgi.is_connected({})'.format(conn_fd))

        # poll for POLLRDHUP on the file descriptor.
        p = select.poll()
        p.register(conn_fd, POLLRDHUP)
        res = p.poll(0)

        for fd, events in res:
            if fd == conn_fd and (events & POLLRDHUP) == POLLRDHUP:
                # the socket peer closed the connection.
                # raise an exception here.
                raise Abort('poll({}, POLLRDHUP)'.format(fd))
except (ImportError, _UnsupportedPlatform):
    def check_connection(): pass
