# Copyright (c) 2012-2015, 2019-2020 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

from collections import OrderedDict
import datetime
import json

def padded_split(s, sep, n):
    ''' Split string into n substrings.
        If there are not n available, return the missing ones as None
    '''
    s = s.split(sep)
    if len(s) < n:
        s.extend([None] * (n - len(s)))
    return s

def iso8601now():
    return datetime.datetime.utcnow().isoformat() + 'Z'

def get_method_args(env):
    s = env.get('REQUEST_URI', None)
    if s:
        # if APIv2 then remap the URL to the old APIv1
        if s[0:10] == '/dnsdb/v2/':
            s = s[10:]
        else:
            raise ValueError("REQUEST_URI does not start with /dnsdb/v2/:", s)

        # REQUEST_URI has the query string appended to it, so we need to remove
        # that part before returning 'args'.
        method, sep, args = s.strip('/').partition('/')
        if '?' in method:
            method = method.partition('?')[0]
        else:
            args = args.partition('?')[0]
        return method, args
    else:
        raise ValueError("REQUEST_URI not available from WSGI environment")

def form_basic_log(env, logtype):
    ''' Forms a basic dnstli log message as an ordered dict.
        After any other fields are added to the dict, log it with: log(NOTICE, json.dumps(logDict))
    '''
    logDict = OrderedDict()
    logDict['timestamp'] = iso8601now()
    logDict['logtype'] = logtype
    logDict['host'] = env.get('SERVER_NAME', 'unset_SERVER_NAME')
    logDict['authtype'] = env.get('dnstli.authtype', 'unset_authtype')
    logDict['authuser'] = env.get('dnstli.authuser', 'unset_authuser')
    logDict['ip'] = env.get('REMOTE_ADDR', 'unset_REMOTE_ADDR')
    if env.get('dnstli.blkq_split'):
        logDict['blkq'] = True

    return logDict
