# Copyright (c) 2016, 2018-2020 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import dnstli2.conf

import dnstli2.response
import dnstli2.util

conf = dnstli2.conf.get_config()

def cors(app):
    '''
    Given an authorization method object, returns a function (f) that
    calls the authorization method object with a passed-in function
    (start_response), passing a wrapper function
    (start_response_add_cors_headers) around start_response
    '''
    def f(env, start_response):
        if 'cors_origins' not in conf:
            origin = '*'
        elif env.get('HTTP_ORIGIN') in conf.get('cors_origins', []):
            origin = env['HTTP_ORIGIN']
        else:
            origin = '{}://{}'.format(env['wsgi.url_scheme'], env['HTTP_HOST'])

        cors_headers1 = [
                ('Access-Control-Allow-Origin', origin),
                ]
        cors_expose = 'Access-Control-Expose-Headers'
        cors_headers2 = [
                ('Access-Control-Max-Age', '86400'),
                ('Access-Control-Allow-Credentials', 'true'),
                ('Access-Control-Allow-Methods', 'GET, POST'),
                ('Access-Control-Allow-Headers', 'Accept, Cache-Control, Pragma, Origin, Content-Type, X-API-Key'),
                ]

        if env['REQUEST_METHOD'] == 'OPTIONS':
            headers_cpy = list(cors_headers1)
            headers_cpy.extend(cors_headers2)
            start_response(dnstli2.response.Raw200Status, headers_cpy)
            return ''

        def start_response_add_cors_headers(status, headers, exc_info=None):
            headers_cpy = list(headers)

            headers_cpy.extend(cors_headers1)

            # Put all headers that start with X-RateLimit into the CORS Expose Headers list
            expose_values = list()
            for header in headers:
                header_left = header[0]
                if header_left.find("X-RateLimit") == 0:
                    expose_values.append(header_left)
            headers_cpy.extend([(cors_expose, ", ".join(expose_values))])

            headers_cpy.extend(cors_headers2)

            return start_response(status, headers_cpy, exc_info)
        return app(env, start_response_add_cors_headers)
    return f
