Running dnstli2 under uwsgi for local development
------------------------------------------------

Note that dnstli2 is only supported under Linux. The steps below assume
you are developing on Debian.

1. Install uWSGI, `sudo apt-get install uwsgi`
    1. An example uWSGI config file is shipped in the `examples/uwsgi/`
        directory.  `localdev_dnstli2.ini` runs the server with API key
        authentication and uses uWSGI's built-in HTTP server to listen on
        `127.0.0.1`.
2. Add [the Farsight apt package repository](https://www.farsightsecurity.com/technical/SIE-user-guide/sie-debian/)
    if it is not added already
3. Install the required packages in the `debian/control` file
4. python3-dnstable and python3-wdns might not install on Buster because they require python3.6
    but Buster comes with python3.7
    1. In this case, clone the source repos: [pydnstable](https://github.com/farsightsec/pydnstable)
        and [pywdns](https://github.com/farsightsec/pywdns)
    2. Install the required packages in their `debian/control` files
    3. Run `sudo python3 setup.py install` for each
5. Install redis with `sudo apt-get install redis`
    1. You can test your redis install with `redis-cli ping`
    2. The response should be "PONG"
6. Add a `/etc/dnstli/dnstli2.conf` file you can copy `examples/dnstli2-basic.conf`
7. Add a /srv/dnstable/mtbl-dns/dns.fileset and some dnstable files
    1. You can grab dnstable data from a stage server like stage-dnsdb-query-1.iad1.fsi.io
    2. Add the filenames of the dnstables you copied to /srv/dnstable/mtbl-dns/dns.fileset
8. Add an api key file to `/etc/dnstli/auth-api`
    1. You can again copy your api key file from a stage server like
        stage-dnsdb-query-1.iad1.fsi.io
9. Run the localdev API server, invoke the following command from the top
    directory of the `dnstli2` working tree:
    ```
    uwsgi --http-socket 127.0.0.1:65080 --plugin syslog --plugin python3 --ini examples/uwsgi/localdev_dnstli2.ini
    ```
10. Test a query with `curl -H "Accept: application/x-ndjson" -H "X-API-Key: $DNSDB_API_KEY" "http://127.0.0.1:65080/dnsdb/v2/lookup/rrset/name/www.farsightsecurity.com?limit=2"`
    1. Alternatively you can use the `dnsdbq` command when the `DNSDB_SERVER`
        variable is set to `"127.0.0.1:65080"`
    2. Make sure to set the DNSDB_API_KEY env variable to the file name you
        added in `/etc/dnstli/auth-api`
11. To get actual results, look for a rrset name in the dnstables you copied using
    [dnstable_dump](https://github.com/farsightsec/dnstable/blob/master/src/dnstable_dump.c)
    and the `--rrset_names` option
    1. Install dnstable_dump by installing the dnstable-bin package `sudo apt-get install dnstable-bin`
    2. If you are not getting any results even with a name in the dnstable,
    make sure the dnstable fileset and dnstable files have the right permissions

The localdev uWSGI config files are configured to send debugging output related
to the application server itself to `stderr`, while the dnstli2 application will
send its own log data to syslog.

For a complete guide to installing on Debian, see the "DNSDB Export Installation for Debian Stretch" document under [Restricted Documentation - DNSDB Export](https://domaintools.atlassian.net/wiki/spaces/SharedServices/pages/3642397379/FSI+Documentation+Repository#DNSDB-Export.1).